#!/usr/bin/perl -W
#
# @file
# Perl based xapian query utility
#
# Development by Lachlan Gunn - http://drupal.org/user/277696
# Polishing and release by Simon Lindsay - http://drupal.org/user/143
#

use strict;
use Search::Xapian;
use DBI;

# database, username, password
my $db = DBI->connect('dbi:mysql:database', 'username', 'password') || die $DBI::errstr;

my $term;

if (scalar(@ARGV) == 0) {
  die 'No query given.';
}

if ($ARGV[0] eq '-n') {
  $term = $ARGV[1];
  shift(@ARGV);
  shift(@ARGV);
}

my $search_db = Search::Xapian::Database->new("xapian_database");

my $parser = Search::Xapian::QueryParser->new();
$parser->set_database($search_db);

my $stemmer = Search::Xapian::Stem->new("english");
$parser->set_stemmer($stemmer);
$parser->set_stemming_strategy(Search::Xapian::STEM_SOME);

my $query = $parser->parse_query(join(' ', @ARGV));
if (defined($term)) {
  $term = Search::Xapian::Query->new($term);
  $query = Search::Xapian::Query->new(Search::Xapian::OP_AND_MAYBE, $term, $query);
}
print "Query: " . $query->get_description() . "\n";

my $enq = Search::Xapian::Enquire->new($search_db);
$enq->set_query($query);

my @matches = $enq->matches(0,10);
print scalar(@matches) . " results\n";

my $sth = $db->prepare('SELECT title FROM node WHERE nid=?');

foreach my $match (@matches) {
  my $doc = $match->get_document();
  my $nid = $doc->get_data();
  $sth->execute($nid);
  if ((my $title) = $sth->fetchrow_array()) {
    printf("ID %d %d%% \t\t--- %d\t%s\n", $match->get_docid(), $match->get_percent(), $nid, $title);
  }
}
